/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c                                                */
/*  DATE        :                                                      */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
#include "iodefine.h"

#define PORTMODE_CONTROL PM7
#define PORTMODE_DATA    PM1
#define PORT_CONTROL     P7
#define PORT_DATA        P1

#define STATE_ON  1
#define STATE_OFF 0

#define INTERNAL_DELAY_TIME 2

#define LINE_FIRST  1
#define LINE_SECOND 4
#define LINE_THIRD  3
#define LINE_FOURTH 2

#define COMMAND_LCD_SET            0x38
#define COMMAND_SCR_CLR            0x01
#define COMMAND_RTN_HOME           0x02
#define COMMAND_DISP_ON            0x0F
#define COMMAND_SCREEN_SHIFT_LEFT  0x18
#define COMMAND_SCREEN_SHIFT_RIGHT 0x1c

void main(void);
void Initialize_Port(void);
void Initialize_LCD(void);

void Instruction_RS(unsigned char bStatus);
void Instruction_RW(unsigned char bStatus);
void Instruction_E(unsigned char bStatus);
void Instruction_E_Pulse(void);

void Command_Direct_LCD(unsigned char nCommand);
void Command_Write_Character(unsigned char cChar);
void Command_Write_String(char* Str);
void Command_Move_Cursor(int xPos,int yPos);
void Command_Print_Line(unsigned char nLineNumber,char* Str);
void Command_Screen_Shift_Left(int nCount,int nDelayTime);
void Command_Screen_Shift_Right(int nCount,int nDelayTime);

void delay_ms(int ms);

void main(void)
{
  Initialize_Port();                   // |[g
  Initialize_LCD();                    // LCD

  Command_Print_Line(LINE_FIRST,"NEWTC, RL78");        // "NEWTC, RL78"\
  Command_Print_Line(LINE_SECOND,"Hello, World!");     // "Hello, World!"\
  Command_Print_Line(LINE_THIRD,"---NAME---");         // "---NAME---"\
  Command_Print_Line(LINE_FOURTH,"RL78 LP Processor"); // "RL78 LP Processor"\

  while (1)
  {

  }

}
void delay_ms(int ms)
{
  volatile int i,j;
  for(i=0;i<ms;i++)
  {
    for(j=0;j<2963;j++)
    {

    }
  }
}
void Initialize_Port(void)
{
  PORTMODE_CONTROL=0x00;               // |[gSo
  PORTMODE_DATA=0x00;                  // f[^|[gSo

  PORT_CONTROL=0x00;                   // |[gSLow
  PORT_DATA=0x00;                      // f[^|[gSLow
}

void Initialize_LCD(void)
{
  Command_Direct_LCD(COMMAND_LCD_SET); // LCDݒ
  Command_Direct_LCD(COMMAND_SCR_CLR); // ʃNA
  Command_Direct_LCD(COMMAND_RTN_HOME);// J[\z[
  Command_Direct_LCD(COMMAND_DISP_ON); // \I
}

void Instruction_RS(unsigned char bStatus)
{
  if(bStatus==STATE_ON)                // ԃI?
  {
    PORT_CONTROL|=0x01;                // RS I
  }
  else
  {
    PORT_CONTROL&=0xFE;                // RS It
  }
}
void Instruction_RW(unsigned char bStatus)
{
  if(bStatus==STATE_ON)                // ԃI?
  {
    PORT_CONTROL|=0x02;                // R/W I
  }
  else
  {
    PORT_CONTROL&=0xFD;                // R/W It
  }
}
void Instruction_E(unsigned char bStatus)
{
  if(bStatus==STATE_ON)                // ԃI?
  {
    PORT_CONTROL|=0x04;                // E I
  }
  else
  {
    PORT_CONTROL&=0xFB;                // E It
  }
}
void Instruction_E_Pulse(void)
{
  Instruction_E(STATE_ON);             // E I
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_E(STATE_OFF);            // E It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
}

void Command_Direct_LCD(unsigned char nCommand)
{
  Instruction_E(STATE_OFF);            // E It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_RS(STATE_OFF);           // RS It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_RW(STATE_OFF);           // R/W It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_E(STATE_ON);             // E I
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  PORT_DATA=nCommand;                  // R}ho
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_E(STATE_OFF);            // E It
}
void Command_Write_Character(unsigned char cChar)
{
  Instruction_E(STATE_OFF);            // E It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_RS(STATE_ON);            // RS I
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_RW(STATE_OFF);           // R/W It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_E(STATE_ON);             // E I
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  PORT_DATA=cChar;                     // f[^o
  Instruction_E(STATE_OFF);            // E It
}
void Command_Write_String(char* Str)
{
  while(*Str!=0)                       // NULL܂ŌJԂ
  {
    Command_Write_Character(*Str);     // o
    Str++;
  }
}
void Command_Move_Cursor(int xPos,int yPos)
{
  Instruction_RS(STATE_OFF);           // RS It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  Instruction_RW(STATE_OFF);           // R/W It
  delay_ms(INTERNAL_DELAY_TIME);       // ԑ҂
  switch(yPos)
  {
  case LINE_FIRST:                     // Ps
    PORT_DATA=xPos+0x80;
    break;
  case LINE_SECOND:                    // Qs
    PORT_DATA=xPos+0xC0;
    break;
  case LINE_THIRD:                     // Rs
    PORT_DATA=xPos+0x94;
    break;
  case LINE_FOURTH:                    // Ss
    PORT_DATA=xPos+0xD4;
  }
  Instruction_E_Pulse();               // EpXo
}
void Command_Print_Line(unsigned char nLineNumber,char* Str)
{
  Command_Direct_LCD(COMMAND_LCD_SET); // LCDZbg
  Command_Move_Cursor(0,nLineNumber);  // J[\ړ
  Command_Write_String(Str);           // 񏑂
}
void Command_Screen_Shift_Left(int nCount,int nDelayTime)
{
  int i;
  for(i=0;i<nCount;i++)
  {
    Command_Direct_LCD(COMMAND_SCREEN_SHIFT_LEFT);  // Vtg
    delay_ms(nDelayTime);              // ԑ҂
  }
}
void Command_Screen_Shift_Right(int nCount,int nDelayTime)
{
  int i;
  for(i=0;i<nCount;i++)
  {
    Command_Direct_LCD(COMMAND_SCREEN_SHIFT_RIGHT);  // EVtg
    delay_ms(nDelayTime);              // ԑ҂
  }
}
